/*************************************************************************
 *                        Boar 's Only A Raytracer                       *
 *                                                                       *
 *       Copyright (C) 2014  Holger Bettag  (hobold@vectorizer.org)      *
 *                                                                       *
 * This program is free software: you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation, either version 3 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * You should have received a copy of the GNU General Public License     *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *************************************************************************/

#ifndef CALC_H
#define CALC_H

#include <math.h>

#include <float.h>
const double MAXNUM = 1e300; // adapt this when changing numerical type

// quick and dirty min, max functions
inline double qmin(double a, double b) {
  return (a < b) ? a : b;
}
inline double qmax(double a, double b) {
  return (a > b) ? a : b;
}

// utility types
struct Vec3 {
  double x;
  double y;
  double z;

  // literal constructor (shortcut for convenient 2-vectors)
  inline Vec3 (double a = 0.0, double b = 0.0, double c = 0.0) {
    x = a;
    y = b;
    z = c;
  };

  // emulate array access to components
  inline double& operator[](int i) const {
    return ((double*)this)[i];
  };

  // vector length
  inline double len(void) const {
    return sqrt(dot(*this));
  };
  
  // scale vector
  inline void scale(double fac) {
    x *= fac;
    y *= fac;
    z *= fac;
  };

  // scale vector, non-destructive
  inline void scale(const Vec3& vec, double fac) {
    x = vec.x * fac;
    y = vec.y * fac;
    z = vec.z * fac;
  };

  // scale vector to given length
  inline void setLen(double Len) {
    double reciLen = Len/len();
    scale(reciLen);
  };

  // scale vector to given length, non-destructive
  inline void setLen(double Len, const Vec3& vec) {
    double reciLen = Len/vec.len();
    x = vec.x*reciLen;
    y = vec.y*reciLen;
    z = vec.z*reciLen;
  };

  // scale vector to unit length
  inline void normalize(void) {
    setLen(1.0);
  };

  // scale vector to unit length, non-destructive
  inline void normalize(const Vec3& vec) {
    setLen(1.0, vec);
  };

  // cross product
  inline void crossOf(const Vec3& a, const Vec3& b) {
    x = a.y * b.z - a.z * b.y;
    y = a.z * b.x - a.x * b.z;
    z = a.x * b.y - a.y * b.x;
  };
  
  // dot product
  inline double dot(const Vec3& vec) const {
    return x*vec.x + y*vec.y + z*vec.z;
  };

  // sum
  inline void add(const Vec3& vec) {
    x += vec.x;
    y += vec.y;
    z += vec.z;
  };

  // difference
  inline void subtract(const Vec3& vec) {
    x -= vec.x;
    y -= vec.y;
    z -= vec.z;
  };
};

#endif // CALC_H
