/*************************************************************************
 *                        Boar 's Only A Raytracer                       *
 *                                                                       *
 *       Copyright (C) 2014  Holger Bettag  (hobold@vectorizer.org)      *
 *                                                                       *
 * This program is free software: you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation, either version 3 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * You should have received a copy of the GNU General Public License     *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *************************************************************************/

#ifndef CAMERA_H
#define CAMERA_H

#include "calc.h"
#include "ray.h"

// virtual base class for ray generators
class Camera {
 public:
  // generate ray corresponding to abstract screen coordinates
  // (coordinates range from 0.0 to 1.0 inclusive, origin is lower left)
  virtual Ray makeRay(double horizontal, double vertical) const = 0;
};

// standard perspective camera (pinhole)
// this camera uses right handed coordinates:
// x -> right, y -> up, z -> towards viewer
class PerspectiveCam : public Camera {
 protected:
  Vec3 eye;        // eye point of camera (origin of all rays)
  Vec3 viewBase;   // world coordinates view rect's lower left corner
  Vec3 viewUp;     // upward pointing edge of view rect (world coordinates)
  Vec3 viewRight;  // right pointing edge of view rect (world coordinates)

  // storage for LookAt values, to facilitate InitRotatedLookAt
  Vec3 lookAt;
  Vec3 sky;
  double width;
  double height;
  double distance;
  Vec3 originalEye;
  
  // camera setup using "look at" parameters
  // internal use only
  void internalLookAt(Vec3 Eye,         // eye point / observer position
		      Vec3 LookAt,      // desired center of image
		      Vec3 Sky,         // reference direction (controls tilt)
		      double Width,     // view rect width
		      double Height,    // view rect height
		      double Distance); // view rect distance (controls FOV);

 public:
  // raw setup
  void Init(Vec3 Eye, Vec3 ViewBase, Vec3 ViewUp, Vec3 ViewRight);

  // camera setup using "look at" parameters
  void InitLookAt(Vec3 Eye,         // eye point / observer position
		  Vec3 LookAt,      // desired center of image
		  Vec3 Sky,         // reference direction (controls tilt)
		  double Width,     // view rect width
		  double Height,    // view rect height
		  double Distance); // view rect distance (controls FOV);

  // for quick and dirty camera flights after InitLookAt()
  // angle in degrees
  void InitRotatedLookAt(double angle);

  // generate ray corresponding to abstract screen coordinates
  // (coordinates range from 0.0 to 1.0 inclusive, origin is lower left)
  virtual Ray makeRay(double horizontal, double vertical) const;
};

#endif // CAMERA_H
