/*************************************************************************
 *                        Boar 's Only A Raytracer                       *
 *                                                                       *
 *       Copyright (C) 2014  Holger Bettag  (hobold@vectorizer.org)      *
 *                                                                       *
 * This program is free software: you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation, either version 3 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * You should have received a copy of the GNU General Public License     *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *************************************************************************/

#ifndef CSG_H
#define CSG_H

#include "geobject.h"
#include "quadric.h"

class CSGObject : public GeObject {
 public:
  virtual VoxClass intersectVoxel(const GeObject*& Obj,
				  const Vec3& Min, const Vec3& Max) const = 0;

  virtual int countPrimitives(void);

  virtual void applyTransform(const Transform& trans);

  virtual void insideOut(void);

  virtual double intersectRay(Ray& Ray, double MinDist,
			      const GeObject*& Primitive) const = 0;
  
  virtual double evaluatePoint(const Vec3& Point) const = 0;
  
  virtual Vec3 surfaceNormal(const Vec3& Point) const = 0;

  virtual Material* getMaterial(void) const;
  
  virtual void setMaterial(const Material* Mat, bool forceReplace = false);

 protected:
  GeObject* child1;
  GeObject* child2;
  int numPrimitives;   // cache total number of primitives in this subtree
};

class Intersection : public CSGObject {
 public:
  Intersection(GeObject* Child1, GeObject* Child2);

  virtual VoxClass intersectVoxel(const GeObject*& Obj,
				  const Vec3& Min, const Vec3& Max) const;

  virtual double intersectRay(Ray& Ray, double MinDist,
			      const GeObject*& Primitive) const;
  
  virtual double evaluatePoint(const Vec3& Point) const;
  
  virtual Vec3 surfaceNormal(const Vec3& Point) const;
};

class Union : public CSGObject {
 public:
  Union(GeObject* Child1, GeObject* Child2);

  virtual VoxClass intersectVoxel(const GeObject*& Obj,
				  const Vec3& Min, const Vec3& Max) const;

  virtual double intersectRay(Ray& Ray, double MinDist,
			      const GeObject*& Primitive) const;
  
  virtual double evaluatePoint(const Vec3& Point) const;
  
  virtual Vec3 surfaceNormal(const Vec3& Point) const;
};

#endif // CSG_H
