/*************************************************************************
 *                        Boar 's Only A Raytracer                       *
 *                                                                       *
 *       Copyright (C) 2014  Holger Bettag  (hobold@vectorizer.org)      *
 *                                                                       *
 * This program is free software: you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation, either version 3 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * You should have received a copy of the GNU General Public License     *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *************************************************************************/

#include "lightsource.h"
#include "trace.h"
#include "simplelist.h"
#include "scene.h"

PointLight::PointLight(Vec3 Position, Vec3 Colour) {
  position = Position;
  colour = Colour;
}

Vec3 PointLight::directionFrom(Vec3 Pos) const {
  Vec3 dir;        // direction vector

  // find direction vector from Pos to PointLight
  dir.x = position.x - Pos.x;
  dir.y = position.y - Pos.y;
  dir.z = position.z - Pos.z;
  
  dir.normalize();
  
  return dir;
}

double PointLight::visibilityFrom(Vec3 Pos, const Scene& scene) const {
  double result = 1.0;
  Ray ray;         // shadow ray
  Vec3 dir;        // direction vector
  double dist;     // distance from Pos to PointLight

  // find direction vector from Pos to PointLight
  dir.x = position.x - Pos.x;
  dir.y = position.y - Pos.y;
  dir.z = position.z - Pos.z;

  // corresponding distance
  dist = dir.len();

  // re-normalize direction vector
  dir.scale(1.0/dist);
  
  // initialize ray
  ray.base = Pos;
  ray.dir = dir;
  
  if (AnyHit(ray, scene, dist) != NULL)
    result = 0.0;
  
  return result;
}
  
Vec3 PointLight::incomingLight(Vec3 Pos) const {
  return colour;
}
