/*************************************************************************
 *                        Boar 's Only A Raytracer                       *
 *                                                                       *
 *       Copyright (C) 2014  Holger Bettag  (hobold@vectorizer.org)      *
 *                                                                       *
 * This program is free software: you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation, either version 3 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * You should have received a copy of the GNU General Public License     *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *************************************************************************/

#ifndef LIGHTSOURCE_H
#define LIGHTSOURCE_H

#include "calc.h"
#include "simplelist.h"

class Scene;  // name declaration

// virtual base class of light sources
class LightSource {
 public:
  // compute normalized vector pointing from Pos to LightPos
  virtual Vec3 directionFrom(Vec3 Pos) const = 0;

  // determine how much of lightsource is seen from Pos
  // 0.0 = not visible, 1.0 = fully visible
  // this is meant to include possible partial occlusion & dimming fx
  // (directional or spot lights may return 0 outside their light beam)
  virtual double visibilityFrom(Vec3 Pos, const Scene& scene) const = 0;
  
  // contribution of fully visible lightsource at Pos
  // this is meant for color changes dependant on distance or direction
  virtual Vec3 incomingLight(Vec3 Pos) const = 0;
};

class PointLight : public LightSource {
 public:
  PointLight(Vec3 Position, Vec3 Colour);

  virtual Vec3 directionFrom(Vec3 Pos) const;

  virtual double visibilityFrom(Vec3 Pos, const Scene& scene) const;
  
  virtual Vec3 incomingLight(Vec3 Pos) const;
  
  // protected:
  Vec3 position;
  Vec3 colour;
};

#endif // LIGHTSOURCE_H
