/*************************************************************************
 *                        Boar 's Only A Raytracer                       *
 *                                                                       *
 *       Copyright (C) 2014  Holger Bettag  (hobold@vectorizer.org)      *
 *                                                                       *
 * This program is free software: you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation, either version 3 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * You should have received a copy of the GNU General Public License     *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *************************************************************************/

#ifndef QUADRIC_H
#define QUADRIC_H

#include "geobject.h"

class Quadric : public PrimObject {
 public:
  inline Quadric(const Vec3& Squared, const Vec3& Mixed, const Vec3& Linear,
		 double Offset) {
    init(Squared, Mixed, Linear, Offset);
  };
  inline Quadric(void) {};
  
  // initializers
  void initSphere(const Vec3& Center, double Radius);
  void initCylinder(const Vec3& Point1, const Vec3& Point2, double Radius);
  void initCone(const Vec3& Point1, const Vec3& Point2,
		double Radius1, double Radius2);
  void initSlab(const Vec3& Point1, const Vec3& Point2);
  void initPlanes(const Vec3& Normal1, double Dist1,
		  const Vec3& Normal2, double Dist2);

  // geometry related methods
  virtual VoxClass intersectVoxel(const GeObject*& Obj,
				  const Vec3& Min, const Vec3& Max) const;

  virtual void applyTransform(const Transform& trans);

  virtual void insideOut(void);

  virtual double intersectRay(Ray& Ray, double MinDist,
			      const GeObject*& Primitive) const;
  
  // intersectRay which additionally returns coefficients of parabola
  // Values[0] = square term
  // Values[1] = linear term
  // Values[2] = constant term
  // Values[3] = near hit (or MAXNUM if invalid)
  // Values[4] = far hit (or MAXNUM if invalid)
  void extractParabola(Ray& Ray, double MinDist,
		       double* Values) const;
  
  virtual double evaluatePoint(const Vec3& Point) const;
  
  virtual Vec3 surfaceNormal(const Vec3& Point) const;

 protected:
  void extractParabolaInternal(Ray& Ray, double MinDist,
			       double* Values) const;

  // internal initializer
  void init(const Vec3& Squared, const Vec3& Mixed, const Vec3& Linear,
	    double Offset);

 public:  // **** TODO: public only for testing and debugging ****
  // master coefficients
  double A, B, C;  // A x^2 + B y^2 + C z^2
  double D, E, F;  // + D yz + E xz + F xy
  double G, H, I;  // + G x + H y + I z
  double J;        // + J
};

#endif // QUADRIC_H
