/*************************************************************************
 *                        Boar 's Only A Raytracer                       *
 *                                                                       *
 *       Copyright (C) 2014  Holger Bettag  (hobold@vectorizer.org)      *
 *                                                                       *
 * This program is free software: you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation, either version 3 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * You should have received a copy of the GNU General Public License     *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *************************************************************************/

#include <stdio.h>
#include <assert.h>

#include "simplelist.h"

// add new data to list (tail)
void List::addItem(void* Data) {
  ListNode* newNode;
  
  newNode = new ListNode(Data);
  
  if (newNode != NULL) {
    if (data == NULL) {
      // empty list requires special case
      assert(next == NULL);  // sanity check for empty list
      next = newNode;
    } else {
      ((ListNode*)data)->append(newNode);
    }
    data = newNode;   // pointer to list tail
  } else {
    fprintf(stderr,"out of memory in List::addItem\n");
    exit(0);
  }
}

// deletes all ListNodes
void List::discard(void) {
  ListNode* current = next;
  ListNode* last;
  
  while (current != NULL) {
    last = current;
    current = current->next;
    delete last;
  }
  next = NULL;
}

// simple comparison of list contents (fragile, just element by element)
bool List::isEqual(List* Reference) const {
  const ListNode* left = firstNode();
  const ListNode* right = Reference->firstNode();
  
  do {
    if (left->getData() != right->getData()) {
      return false;  // differing data => not equal
    }

    left = left->nextNode();
    right = right->nextNode();
    
    if ((left == NULL) && (right == NULL)) {
      return true;  // same data and same length => equal
    }
    
    if ((left == NULL) || (right == NULL)) {
      return false;  // different length => not equal
    }
    
  } while (true);

  return true;
}
