/*************************************************************************
 *                        Boar 's Only A Raytracer                       *
 *                                                                       *
 *       Copyright (C) 2014  Holger Bettag  (hobold@vectorizer.org)      *
 *                                                                       *
 * This program is free software: you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation, either version 3 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * You should have received a copy of the GNU General Public License     *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *************************************************************************/

#ifndef SIMPLELIST_H
#define SIMPLELIST_H

// for NULL
#include <stdlib.h>

// quick & dirty simplistic linked list
class List;

class ListNode {
  friend class List;
  
 public:
  ListNode(void) {
    next = NULL;
  };
  
  ListNode(void* Data) {
    data = Data;
    next = NULL;
  };

  inline const ListNode* nextNode(void) const {
    return next;
  };
  
  inline void* getData(void) const {
    return data;
  };
  

 protected:
  // insert new node after this node
  inline void append(ListNode* newNode) {
    newNode->next = next;
    next = newNode;
  };
  
  ListNode* next;
  void* data;
};

// simply a special list node which is always a list head
// (the embedded data field points to the last node, next to the first node)
class List : public ListNode {
 public:
  inline List(void) {
    next = NULL;
    data = NULL;
  };

  // destructor does not interpret user data pointers
  // (those need to be deleted by the list user)
  inline ~List() {
    discard();
  };

  // deletes all ListNodes
  void discard(void);
  

  inline const ListNode* firstNode(void) const {
    return next;
  };

  // unsorted insertion of new node (make no assumptions about order)
  inline void add(ListNode* newNode) {
    this->append(newNode);
  };

  // creation of new list entry with specified user data
  // (insertion at list tail)
  void addItem(void* Data);

  // simple comparison of list contents (fragile, just element by element)
  bool isEqual(List* Reference) const;
};

#endif // SIMPLELIST_H
